@@ -1,13 +1,13 @@
 # flake8: noqa
 """Data collection manager"""
-from .config_class import AswanConfig, EnvConfig
 from .connection_session import ConnectionSession
 from .constants import ONE_DAY, ONE_HOUR, ONE_MINUTE, ONE_WEEK, ONE_YEAR, Statuses
+from .depot import AswanDepot
 from .exceptions import BrokenSessionError, ConnectionError
+from .object_store import ObjectStore
 from .project import Project
-from .security.proxy_base import ProxyAuth, ProxyBase
-from .simplified_functions import get_json, get_soup
-from .t2_integrators import FlexibleDfParser, T2Integrator
+from .security import ProxyAuth, ProxyBase
+from .simplified_functions import get_json, get_soup, run_simple_project
 from .url_handler import (
     BrowserHandler,
     BrowserJsonHandler,
@@ -17,4 +17,4 @@ from .url_handler import (
     RequestSoupHandler,
 )
 
-__version__ = "0.2.0"
+__version__ = "0.3.1"
