@@ -20,15 +20,16 @@ def test_fun_logger(capsys):
     assert d["k"] == 10
 
 
-def test_url_param_add():
+@pytest.mark.parametrize(
+    ("params", "suffix"),
+    [({"k": "v"}, "?k=v"), ({"k1": 1, "k2": "v2"}, "?k1=1&k2=v2")],
+)
+def test_url_param_add(params, suffix):
+    root = "http://test.com"
+    assert add_url_params(root, params) == f"{root}{suffix}"
 
-    assert (
-        add_url_params("http://test.com", {"k": "v"}) == "http://test.com?k=v"
-    )
-    assert (
-        add_url_params("http://test.com", {"k1": "v1", "k2": "v2"})
-        == "http://test.com?k1=v1&k2=v2"
-    )
+
+def test_extend_url_params():
     assert (
         add_url_params("http://test.com?k1=v1", {"k2": "v2", "k3": "v3"})
         == "http://test.com?k1=v1&k2=v2&k3=v3"
@@ -52,8 +53,5 @@ def test_url_root(full_url):
 def test_url_root_weird():
 
     assert get_url_root("http://te-st.com/a/b") == "http://te-st.com"
-    assert (
-        get_url_root("https://xx.re.te-st.com/a/b")
-        == "https://xx.re.te-st.com"
-    )
+    assert get_url_root("https://xx.re.te-st.com/a/b") == "https://xx.re.te-st.com"
     assert get_url_root("nothing") is None
